/*
 *     Botendo
 *     Copyright (C) 2023  limited_dev
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

package de.limited_dev.botendo

import de.limited_dev.botendo.commands.slash.component.SlashCommandManager
import de.limited_dev.botendo.commands.slash.component.options.CommandOption
import de.limited_dev.botendo.commands.slash.component.options.OptionType
import de.limited_dev.botendo.util.Logger
import de.limited_dev.botendo.util.TokenManager
import dev.kord.core.Kord
import dev.kord.core.event.interaction.GuildChatInputCommandInteractionCreateEvent
import dev.kord.core.on
import dev.kord.gateway.Intent
import dev.kord.gateway.PrivilegedIntent
import dev.kord.rest.builder.interaction.boolean
import dev.kord.rest.builder.interaction.integer
import dev.kord.rest.builder.interaction.string

object Bot {
    //The kord object gets set at app launch
    var kord: Kord? = null
    suspend fun start() {
        Logger.out("Starting Bot...")

        //Load config
        TokenManager.load()

        //Don't run the bot when there is no bot token in config
        if (TokenManager.token == "empty") {
            Logger.out("The config does not contain a bot token")
            return
        }

        //Register slash commands to bot & Discord
        SlashCommandManager.register()

        //Add Bot token to kord
        kord = Kord(TokenManager.token)

        //Register Command Listener
        kord!!.on<GuildChatInputCommandInteractionCreateEvent> {
            val response = interaction.deferPublicResponse()
            val command = interaction.command
            Logger.out("Command /${command.rootName}")
            for (c in SlashCommandManager.commands) {
                if (c.name != command.rootName)
                    continue
                val opt = mutableMapOf<CommandOption, String>()
                if (c.options != null) {
                    for (o in c.options) {
                        when (o.type) {
                            OptionType.INT ->
                                if (command.integers[o.name] != null)
                                    opt[o] = command.integers[o.name].toString()

                            OptionType.STRING ->
                                if (command.strings[o.name] != null)
                                    opt[o] = command.strings[o.name].toString()

                            OptionType.BOOLEAN ->
                                if (command.booleans[o.name] != null)
                                    opt[o] = command.booleans[o.name].toString()
                        }
                    }
                }
                c.onSlashCommand(interaction, response, opt)
                //response.respond {  }
            }
        }

        //Register Slash Commands at Discord
        kord!!.createGlobalApplicationCommands {
            for (sc in SlashCommandManager.commands) {
                this.input(sc.name, sc.description, builder = {
                    if (sc.options != null) {
                        for (o in sc.options) {
                            when (o.type) {

                                OptionType.INT -> {
                                    integer(o.name, o.description) {
                                        this.required = o.required
                                    }
                                }

                                OptionType.STRING -> {
                                    string(o.name, o.description) {
                                        this.required = o.required
                                    }
                                }

                                OptionType.BOOLEAN -> {
                                    boolean(o.name, o.description) {
                                        this.required = o.required
                                    }
                                }
                            }
                        }
                    }
                })
            }
        }

        Logger.out("Logging in")

        //Start the bot
        kord!!.login {
            // Load gateway privileges
            @OptIn(PrivilegedIntent::class)
            intents += Intent.GuildMembers

            @OptIn(PrivilegedIntent::class)
            intents += Intent.MessageContent
        }
    }
}
